#!/usr/bin/env python
#
# -------------------------------------------------------------------------------------
#
# Copyright (c) 2020 Skinnyhouse
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification, 
# are permitted provided that the following conditions are met:
#
#    - Redistributions of source code must retain the above copyright notice, this 
#    list of conditions and the following disclaimer.
#    - Redistributions in binary form must reproduce the above copyright notice, 
#    this list of conditions and the following disclaimer in the documentation and/or 
#    other materials provided with the distribution.
#    - Neither the name of the author nor the names of its contributors may be used 
#    to endorse or promote products derived from this software without specific prior 
#    written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY 
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
# SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
# TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
# DAMAGE.
#
# -------------------------------------------------------------------------------------
#
# This file is a basic Python plug-in for GIMP used to apply version 1.7 of the Bevel Reflect Logo script to each of the character layers in a custom font file.
# It simply applies the work principally of GnuTux (2013) and MareroQ (2020) in batch mode.
#
# It can be executed by selecting the menu option: 'Filters>>Custom Font Tools>>Custom Font Bevel Reflect...

# v0.00 This is the initial release
# It obviously needs to access the Bevel Reflect Logo script and the associated application in order to work
# The filter calls the Bevel Reflect Logo script and applies the effect to each of the character layers in a custom font file.
# Most of the options available from the user dialog mirror the options available in the original script dialog
# The following options are exclusive to this filter:
#    Option to trim pixels from the character should the user need to tidy up the character edges
#    Option to desaturate output from BRL where coloured gradients have been used to create a topological map and are to be applied to an existing custom font
#    NB Should the user want to apply (gradient/foreground/background) colour to a plain character, better results will be achieved by using a mid-grey plain coloured custom font instead of the usual white characters



from gimpfu import *
#from array import array
#import math
#import random

#from gtk import gdk
#w = gdk.screen_width()
#h = gdk.screen_height()

# Some global colours we may need to define

black = gimpcolor.RGB(0, 0, 0)
white = gimpcolor.RGB(255, 255, 255)
#darkgrey = gimpcolor.RGB(50, 50, 50)
#midgrey = gimpcolor.RGB(125, 125, 125)
#lightgrey = gimpcolor.RGB(170, 170, 170)
#verylightgrey = gimpcolor.RGB(222, 222, 222)
#red = gimpcolor.RGB(220,10,10)	



def do_stuff(image, layer, trim_value, desaturate_option, slope_options, flat_bevel_smoothing, flat_bevel_depth, round1_bevel_depth, round2_bevel_depth, diffuse_shiny, contour, contour_depth, edge_smoothing, emap_name, environment_map_type, blending_mode, blend_direction, blend_shape, light_direction, specular_highlight, detail_level, horizontal_texture_size, vertical_texture_size, emap_final_blur):
    #### Main Filter Proc
	
    # Store the GIMP's settings so they can be restored
    gimp.context_push()
    # Set start undo point
    pdb.gimp_image_undo_group_start(image)
	
	
    plugin = False
    num_plugins, menu_path, num_plugins, plugin_accelerator, num_plugins, plugin_location, num_plugins, plugin_imgtype, num_plugins, plugin_time, num_plugins, plugin_real_name = pdb.gimp_plugins_query("reflect")
    
    if "bevelreflect2" in str(plugin_real_name): plugin = True
    #pdb.gimp_message(str(plugin_real_name) + str(plugin) )
    
    loop_ptr = 1
    first_char_layer_name = "none"
    num_layers, layer_ids = pdb.gimp_image_get_layers(image)
    loop = num_layers - loop_ptr
    layer_names_list = image.layers
    original_active_layer = image.active_layer
	
	
    visi_loop, first_char_layer  = 0, -1
    other_layers,num_char_layers = 0,0
    apply_to_custom = False
	
	
    #pdb.gimp_context_set_background(black)
    #pdb.gimp_context_set_foreground(white)	
	
	
    while visi_loop < num_layers:
	    # get info about layer we are working on
        layer_name = pdb.gimp_item_get_name(layer_names_list[visi_loop])
        layer_name = layer_name.decode('utf-8')
		
		
        test_for_custom_layer = fn_test_for_custom_layer(image, layer_names_list[visi_loop], layer_name)
			
        if test_for_custom_layer:
		
            apply_to_custom = True
			
            num_char_layers += 1
            if first_char_layer < 0: 
                first_char_layer = visi_loop
                first_char_layer_name = layer_name
                layer_names_list[visi_loop].set_offsets(0, 0)
		
        else:
                
            face_name = pdb.gimp_item_get_name(layer_names_list[visi_loop])
            if other_layers == 0: other_layer_list = [face_name]
		
            other_layers += 1
			
        pdb.gimp_item_set_visible(layer_names_list[visi_loop],False) # turn off visibility
        visi_loop += 1
			
			
			
    if not apply_to_custom and plugin == True:	
        message_text = "No custom font character layers."
        old_handler = pdb.gimp_message_get_handler()	
        pdb.gimp_message_set_handler(MESSAGE_BOX)
        pdb.gimp_message(message_text)
        pdb.gimp_message_set_handler(old_handler)
		
    if not plugin:
        message_text = "BevelReflect2 plugin cannot be found.\nUnable to continue."
        apply_to_custom = False
        old_handler = pdb.gimp_message_get_handler()	
        pdb.gimp_message_set_handler(MESSAGE_BOX)
        pdb.gimp_message(message_text)
        pdb.gimp_message_set_handler(old_handler)
		
			
    if apply_to_custom:
        # set some plug-in variables
        # those that are commented out are taken from the user dialog
        active_to_new = False
        #slope_options = 0
        #flat_bevel_smoothing = 8.0
        #flat_bevel_depth = 10.0
        #round1_bevel_depth = 7.0
        #round2_bevel_depth = 20
        #diffuse_shiny = 1.00
        #contour = 0
        #contour_depth = 0
        #edge_smoothing = 1
        #emap_name = "none"
		
        if not (emap_name.endswith(".jpg") or emap_name.endswith(".JPG") or emap_name.endswith(".png") or emap_name.endswith(".PNG")):
            emap_name = "none"
        if emap_name == "": emap_name = "none"
            
        #environment_map_type = 0
        #blending_mode = 0
        #blend_direction = 0
        #blend_shape = 0
        #bevel_gradient = "FG to BG (RGB)"
        bevel_gradient = pdb.gimp_context_get_gradient()
        #light_direction = 315
        #specular_highlight = 0.35
        #detail_level = 0
        #horizontal_texture_size = 1.2
        #vertical_texture_size = 2.6
        emap_final_size = 600
        #emap_final_blur = 0
        auto_stretch_HSV = True
        turbulent_noise = True
        keep_layers = False
		
				
    while loop >= 0 and apply_to_custom:
	    # get info about layer we are working on for custom font mode
        layer_id = layer_names_list[loop]
        layer_name = pdb.gimp_item_get_name(layer_names_list[loop])
		
        #test_for_custom_layer = fn_test_for_custom_layer(image, layer_names_list[loop], layer_name)
	
        # cater for unicode characters above normal range
        display_layer_name =  pdb.gimp_item_get_name(layer_names_list[loop])
        display_layer_name = display_layer_name.decode('utf-8')	
	
        # check we have the layer id of the currently active layer
        pdb.gimp_image_set_active_layer(image, layer_id)
        layer_id = pdb.gimp_image_get_active_layer(image)
        active_layer_id = pdb.gimp_image_get_active_layer(image)


			
				
        test_for_custom_layer = fn_test_for_custom_layer(image, layer_id, layer_name)
			
        if test_for_custom_layer:
            # ensure we have the active layer and make it visible
            active_layer_id = pdb.gimp_image_get_active_layer(image)
            pdb.gimp_item_set_visible(active_layer_id, True)

            active_layer_id = proc_bevel_reflect(image, active_layer_id, active_to_new, slope_options, flat_bevel_smoothing, flat_bevel_depth, round1_bevel_depth, round2_bevel_depth, diffuse_shiny, contour, contour_depth, edge_smoothing, emap_name, environment_map_type, blending_mode, blend_direction, blend_shape, bevel_gradient, light_direction, specular_highlight, detail_level, horizontal_texture_size, vertical_texture_size, emap_final_size, emap_final_blur, auto_stretch_HSV, turbulent_noise, keep_layers, trim_value, desaturate_option)
				
				
        if (num_layers - loop_ptr) > 0: pdb.gimp_item_set_visible(active_layer_id, False)
			
        num_layers, layer_ids = pdb.gimp_image_get_layers(image)
        pdb.gimp_image_resize_to_layers(image)
			
        loop_ptr += 1
        loop = num_layers - loop_ptr
		
		
	
    if first_char_layer_name <> "none": # the character layer to make visible on completion
        first_char_layer_id = pdb.gimp_image_get_layer_by_name(image, first_char_layer_name)
        pdb.gimp_item_set_visible(first_char_layer_id, True)
        pdb.gimp_image_set_active_layer(image,first_char_layer_id)
    else:	
        active_layer = image.active_layer
        pdb.gimp_item_set_visible(original_active_layer, True)
        pdb.gimp_image_set_active_layer(image,original_active_layer)
	
	
	
    # Set end undo point
    pdb.gimp_image_undo_group_end(image)

    # Restore the GIMP's settings
    gimp.context_pop()	
	
######################################################################################################################################

def proc_bevel_reflect(image, active_layer_id, active_to_new, slope_options, flat_bevel_smoothing, flat_bevel_depth, round1_bevel_depth, round2_bevel_depth, diffuse_shiny, contour, contour_depth, edge_smoothing, emap_name, environment_map_type, blending_mode, blend_direction, blend_shape, bevel_gradient, light_direction, specular_highlight, detail_level, horizontal_texture_size, vertical_texture_size, emap_final_size, emap_final_blur, auto_stretch_HSV, turbulent_noise, keep_layers, trim_value, desaturate_option):

    # get character layer name for later rename
    character_name = pdb.gimp_item_get_name(active_layer_id)
    # make a copy of the character layer and add the layer to the image
    position = pdb.gimp_image_get_item_position(image, active_layer_id)
    temp_copy = pdb.gimp_layer_copy(active_layer_id, True)
    image.add_layer(temp_copy, position)
	
    pdb.gimp_item_set_name(temp_copy,"Temp Copy") # rename the image
    pdb.gimp_item_set_visible(temp_copy, True)
    #call bevel reflect script
    pdb.script_fu_ml_bevel_reflect_logo_alpha(image, active_layer_id, active_to_new, slope_options, flat_bevel_smoothing, flat_bevel_depth, round1_bevel_depth, round2_bevel_depth, diffuse_shiny, contour, contour_depth, edge_smoothing, emap_name, environment_map_type, blending_mode, blend_direction, blend_shape, bevel_gradient, light_direction, specular_highlight, detail_level, horizontal_texture_size, vertical_texture_size, emap_final_size, emap_final_blur, auto_stretch_HSV, turbulent_noise, keep_layers)
    # identify topological layer generated by the call to the script and remove it
    br_layer = pdb.gimp_image_get_layer_by_name(image, "BR Logo")
    pdb.gimp_image_set_active_layer(image, br_layer)
    if trim_value > 0: proc_trim_character(image, br_layer, trim_value) # allow user to trim pixels from the selection outline
    # allow user to desaturate the layer generated by the BRL plug-in before applying mode effect
    if desaturate_option > 0: pdb.gimp_drawable_desaturate(br_layer, desaturate_option-1)
	
    mode = 47 # grain merge
    temp_copy.mode = mode # apply mode
    temp_copy = pdb.gimp_image_merge_down(image, temp_copy, 0) # merge down the copy of the character layer
    pdb.gimp_item_set_name(temp_copy,character_name) # reinstate the original name of the character
	
    # get the name of the topological map layer generated for each map type by BRL
    if environment_map_type == 0:
        topo_layer = pdb.gimp_image_get_layer_by_name(image, "Spherical Topo")
    elif environment_map_type == 1:
        topo_layer = pdb.gimp_image_get_layer_by_name(image, "Spherical Cloud")
    elif environment_map_type == 2:
        topo_layer = pdb.gimp_image_get_layer_by_name(image, "Spherical Topo")
    elif environment_map_type == 3:
        topo_layer = pdb.gimp_image_get_layer_by_name(image, "Spherical Cloud")
    elif environment_map_type == 4:
        topo_layer = pdb.gimp_image_get_layer_by_name(image, (character_name + " copy"))
		
    if emap_name == "none": pdb.gimp_image_remove_layer(image, topo_layer) # and remove the topological map layer

    active_layer_id = pdb.gimp_image_get_active_layer(image) # identify the new active layer

    return active_layer_id
			

######################################################################################################################################

def fn_test_for_custom_layer(img,l_id,l_name):
    # checks that the current layer is compatible with custom fonts
    test = False
    if len(l_name) == 1:
        test = True
    elif len(l_name) == 5 or len(l_name) == 6:
        
        if ".png" in l_name:
            test = True
            if len(l_name) == 6:
                part_name_val = unicode(l_name[0:2])
                if part_name_val <= 65535: test = False
    # select the contents of the layer
    pdb.gimp_image_select_item(img,2,l_id)
    # Test to see if the selection actually contains something
    sel_is_empty = pdb.gimp_selection_is_empty(img)
    # If there are no pixels to work with flag the layer as not viable to work on
    if sel_is_empty: test = False
    # Deselect
    pdb.gimp_selection_none(img)
    return test
	
######################################################################################################################################

    #### Useful Functions
# -------------------------------------------------------------------------------------
	
def fn_selection_info(im):
    # returns selection info
    selection,x1,y1,x2,y2 = pdb.gimp_selection_bounds(im)
    s_width, s_height = x2 - x1, y2 - y1
	
    return selection, x1, x2, y1, y2, s_width, s_height


######################################################################################################################################
	
def fn_selection_amount(sel_value,r_ness,adjustment):
    # 
    value = (math.sqrt(sel_value)*r_ness)*adjustment
    return value

######################################################################################################################################
	
def proc_messagebox(message):
    old_handler = pdb.gimp_message_get_handler()
    pdb.gimp_message_set_handler(2)
    pdb.gimp_message(str(message))
    pdb.gimp_message_set_handler(old_handler)

######################################################################################################################################
	
def proc_trim_character(img, layer, amount):    
    # select the contents of the layer
    pdb.gimp_image_select_item(img, 2, layer)
    # shrink the selection
    pdb.gimp_selection_shrink(img, amount)
    # invert selection
    pdb.gimp_selection_invert(img)
    # trim away pixels
    successful = pdb.gimp_edit_cut(layer)
    # deselect
    pdb.gimp_selection_none(img)
	
######################################################################################################################################
		
def proc_trim_away(img, layer, amount):
    #### Trim off pixels from a layer
    # set our feather settings
    our_feather = True
    our_feather_radius = 2
    #get feather settings
    feather = pdb.gimp_context_get_feather()
    feather_x, feather_y = pdb.gimp_context_get_feather_radius()
    # make sure that feather is set
    if not feather: pdb.gimp_context_set_feather(our_feather)
    # use our feather radius
    pdb.gimp_context_set_feather_radius(our_feather_radius, our_feather_radius)
    # select the contents of the layer
    pdb.gimp_image_select_item(img, 2, layer)
    # shrink the selection
    pdb.gimp_selection_shrink(img, amount)
    # invert selection
    pdb.gimp_selection_invert(img)
    # trim away pixels
    successful = pdb.gimp_edit_cut(layer)
    # deselect
    pdb.gimp_selection_none(img)
    # reset feather settings
    pdb.gimp_context_set_feather(feather)
    pdb.gimp_context_set_feather_radius(feather_x, feather_y)
	
    #### End of Proc_trim_away
	
######################################################################################################################################
	
register(
    "python_fu_Custom_Font_Bevel_Reflect",
    "Applies the Bevel Reflect Logo script to custom font characters",
    "Applies the Bevel Reflect Logo script to custom font characters",
    "Skinnyhouse",
    "Open source (BSD 3-clause license)",
    "2020",
    "<Image>/Filters/Custom Font Tools/Custom Font Bevel Reflect...",
    "RGB*",
    [
    (PF_SPINNER, "pf_shine_char", "Trim character in pixels: ", 0,(0,10,1)),
    (PF_OPTION, "desaturate_option", "Desaturate emap result options", 0, ("None applied", "Desaturate Lightness", "Desaturate Luma", "Desaturate Average", "Desaturate Luminance", "Desaturate Value")),
    (PF_OPTION, "Slope_option", "Slope Options", 0, ("Flat Bevel", "Round1 Bevel", "Round2 Bevel")),
    (PF_SPINNER, "Flat_bevel_smoothing", "Flat Bevel - Smoothing", 8.0, (1.0, 32.0, 1.0)),
    (PF_SPINNER, "Flat_bevel_depth", "Flat Bevel - Depth", 10.0, (1.0, 10.5, 0.5)),
    (PF_SPINNER, "Round1_depth", "Round1 Bevel - Depth", 7.0, (1.0, 8.0, 0.5)),
    (PF_SPINNER, "Round2_depth", "Round2 Bevel - Depth", 20.0, (1.0, 25.0, 0.5)),
    (PF_SPINNER, "Diffuse_Shiny", "(Diffuse) 0<--->1 (Shiny)", 1.0, (0.0, 1.0, 0.1)),
    (PF_OPTION, "Contour_option", "Contour Options", 0, ("Linear", "Outline", "Outline (Inner Bevel)", "Outline (Outer Bevel)", "Step Bevel", "Ring (Narrow)", "Ring (Medium)", "Ring (Wide)", "Ring (Extra Wide)", "Ring (Thick)", "Double Ring", "Double Ring (Narrow)", "Double Ring (Wide)")),
    (PF_SPINNER, "Contour_depth", "Contour Depth", 0, (0, 20, 1)),
    (PF_SPINNER, "Edge_smoothing", "Edge Smoothing Amount", 1, (0, 4, 1)),
    (PF_FILENAME, "pf_emap_name", "Spherical Emap (png/jpg):", "none"),
    (PF_OPTION, "Environment_Map_Type", "Environment Map Type", 0, ("Spherical Topographical", "Spherical Cloud", "Spherical Topo (Active Gradient)", "Spherical Cloud (Active Gradient)", "Spherical Solid (Active Color)")),
    (PF_OPTION, "Blending_Mode", "Blending Mode", 0, ("Normal", "Dissolve", "Behind", "Multiply", "Screen", "Overlay", "Difference", "Addition", "Subtract", "Darken", "Lighten", "Hue", "Saturation", "Color", "Value", "Divide", "Dodge", "Burn", "Hard Light", "Soft Light", "Grain Extract", "Grain Merge")),
    (PF_OPTION, "Blend_Direction", "Blend Direction", 0, ("Top-Botton", "Bottom-Top", "Left-Right", "Right-Left", "Diag-Top-Left", "Diag-Top-Right", "Diag-Bottom-Left", "Diag-Bottom-Right")),
    (PF_OPTION, "Blend_Shape", "Blend Shape", 0, ("Linear", "Bilinear", "Radial", "Square", "Conical-Symmetric", "Conical-Asymmetric", "Shapeburst-Angular", "Shapburst-Spherical", "Shapeburst-Dimpled", "Spiral-CW", "Spiral-CCW")),
    (PF_SPINNER, "Light_Direction", "Light Direction (Degrees)", 315, (0, 360, 45)),
    (PF_SPINNER, "Emap_Specular_Highlights", "Emap Specular Highlights", 0.35, (0.10, 0.65, 0.05)),
    (PF_SPINNER, "Detail_Level", "Detail Level", 0, (0, 3, 1)),
    (PF_SPINNER, "Horizontal_Texture_Size", "Horizontal Texture Size", 1.2, (1.0, 5.0, 0.1)),
    (PF_SPINNER, "Vertical_Texture_Size", "Vertical Texture Size", 2.6, (1.0, 5.0, 0.1)),
    (PF_SPINNER, "EMap_Final_Blur", "EMap Final Blur", 0, (0, 20, 1)),
    #(PF_SLIDER, "pf_azimuth", "Shine Azimuth:", 315, (0, 359, 1)),
    #(PF_TOGGLE, "pf_usecustomfont", "Combine the current character \nwith one from the custom font below:", 0),
    #(PF_FILENAME, "pf_names_xcffilename", ".xcf custom font file:", 0),
    #(PF_INT, "number", "Number?", 50),
    #(PF_FLOAT, "angle", "Angle", 3.14159),
    #(PF_STRING, "word", "Word", "Zebrafish!"),
    #(PF_TEXT, "text", "Some Text",
    #(PF_RADIO, "imagefmt", "Image format", "jpg", (("png", "png"), ("jpg", "jpg"))),
    #(PF_OPTION, "option", "Option", 2, ("Mouse", "Cat", "Dog", "Horse")),
    #(PF_SPINNER, "size", "Pixel Size", 50, (1, 8000, 1)),
    #(PF_FONT, "font", "Font", "Sans"),
    #(PF_BRUSH, "brush", "Brush", None),
    #(PF_PATTERN, "pattern", "Pattern", None),
    #(PF_GRADIENT, "gradient", "Gradient", None),
    #(PF_PALETTE, "palette",  "Palette", ""),
    #(PF_VECTORS, "vectors", "Vectors", None),
    ],
    [],
    do_stuff)

main()
